//--------------------------------------------------------------------  //000
//                                                                      //001
//  Modul test                                                          //002
//                                                                      //003
//  Testumgebung fuer den Display-Controller                            //004
//                                                                      //005
//--------------------------------------------------------------------  //006
                                                                        //007
`include "discount_defs.v"

module test;                                                            //008
                                                                        //009
  `define Simtime ((`Hline)*(`Vframe)*120)  // 1.2 Frames simulieren    //010
                                                                        //011
  // Deklarationen                                                      //012
  reg [`Bsz-1:0]  MEM[0:`Msz-1];  // Bildspeicher                       //013
  reg [`Bsz-1:0]  TEMP;           // Hilfsvariable (Speicher)           //014
  reg [`Dsz-1:0]  DATA;           // Bildspeicherausgang                //015
  reg             CLOCK,          // Takt                               //016
                  NRST,           // Initialisierung                    //017
					   SNAPSHOT;       // Bild speichern (PBM)
  wire [`Asz-1:0] ADDR;           // Bildspeicheradresse                //018
  wire            PIXEL,          // Bildpunkt                          //019
                  CSYNC,          // kombinierte Synchronisation        //020
                  HSYNC,          // horizontale Synchronisation        //021
                  VSYNC;          // vertikale Synchronisation          //022
  integer         BYTE, BIT,      // Speicherposition                   //023
                  H, V;           // Bildposition                       //024
                                                                        //025
  //                                                                    //026
  // Testinstanz des Display-Controllers                                //027
  //                                                                    //028
  discount DISCOUNT (ADDR,PIXEL,CSYNC,HSYNC,VSYNC,DATA,CLOCK,NRST);     //029
                                                                        //030
  //                                                                    //026
  // Instanz des Testmonitors (schreibt PBM)
  //
  crt CRT (PIXEL,CSYNC,HSYNC,VSYNC,SNAPSHOT,CLOCK);

  //                                                                    //031
  // Speicher initialisieren, weisse Diagonale                          //032
  //                                                                    //033
  initial begin                                                         //034
    BYTE = 0;                             // Initialisierung            //035
    BIT = `Bsz-1;                                                       //036
    TEMP = MEM[0];                                                      //037
    for (V = 0; V < `V; V = V+1)          // Zeilen                     //038
      for (H = 0; H < `H; H = H+1) begin  // Punkte                     //039
        if (V == H) TEMP[BIT] = 1;        // Diagonale                  //040
        else        TEMP[BIT] = 0;                                      //041
        if (BIT == 0) begin               // neues Byte                 //042
          MEM[BYTE] = TEMP;                                             //043
          if (BYTE < `Msz-1)              // nicht letztes Byte         //044
            BYTE = BYTE+1;                                              //045
          TEMP = MEM[BYTE];                                             //046
          BIT = `Bsz-1;                                                 //047
        end                                                             //048
        else BIT = BIT-1;                                               //049
      end                                                               //050
  end                                                                   //051
                                                                        //052
  //                                                                    //053
  // Takt erzeugen                                                      //054
  //                                                                    //055
  always begin                                                          //056
    CLOCK = 1'b0;                                                       //057
    #50;                                                                //058
    CLOCK = 1'b1;                                                       //059
    #50;                                                                //060
  end                                                                   //061
                                                                        //062
  //                                                                    //063
  // Reset                                                              //064
  //                                                                    //065
  initial begin                                                         //066
    NRST = 1'b0;                                                        //067
    @(negedge CLOCK);                                                   //068
    NRST = 1'b1;                                                        //069
  end                                                                   //070
                                                                        //071
  //                                                                    //072
  // Bildspeicherdaten auslesen                                         //073
  //                                                                    //074
  always @(ADDR) begin                                                  //075
    DATA = MEM[ADDR];                                                   //076
  end                                                                   //077
                                                                        //078
  initial                                                               //079
    #`Simtime $stop;                    // Simulationsdauer             //080
                                                                        //081
  //                                                                    //103
  // Textausgabe                                                        //104
  //                                                                    //105
  initial                                                               //106
    $monitor("Takt=%d  PIXEL=%b  HSYNC=%b  VSYNC=%b",                   //107
      $time/100, PIXEL, HSYNC, VSYNC);                                  //108
                                                                        //109
  //                                                                    //110
  // Diagonale verifizieren                                             //111
  //                                                                    //112
  always @(posedge CLOCK) begin                                         //113
    H = DISCOUNT.HCOUNT.HCNT-`H1-1;      // Bildposition                //114
    V = DISCOUNT.VCOUNT.VCNT-`V1;                                       //115
                                                                        //116
    if ((H>=0) && (H< `H) &&             // echter Bildpunkt            //117
        (V> 0) && (V<=`V))                                              //118
                                                                        //119
      if (((H==V) && (PIXEL==0)) ||      // Diagonale                   //120
          ((H!=V) && (PIXEL==1)))        // ausserhalb Diagonale        //121
        $display(                                                       //122
        "falsche Diagonale bei Takt %0d, PIXEL = %b, H = %0d, V = %0d"  //123
        ,$time/100, PIXEL, H, V);                                       //124
  end                                                                   //125

  //
  // Snapshot speichern
  //
  initial
    SNAPSHOT = 1;
                                                                        //126
endmodule // test                                                       //127
                                                                        //128